/**
 * File for project logging.
 * Contains actual logic for class defined in logging.h.
 */


// User Import Headers.
#ifndef user_headers
    #define user_headers
    #define user_headers_logging
    #include "logging.h"
#endif


/**
 * Class constructor.
 */
Logging::Logging() {
    std::filesystem::path cwd = std::filesystem::current_path();
    core_logging_folder = cwd.string() + "/log/";

    // Check if logging directory exists.
    struct stat info;
    if ( (stat(core_logging_folder.c_str(), &info) != 0) || (! info.st_mode & S_IFDIR)) {
        // Directory does not exist.
        mkdir(core_logging_folder.c_str(), 0777);
    }

    // Open file descriptors.
    error_stream.open(core_logging_folder + "error.log", std::ios::app | std::ios::out);
    warn_stream.open(core_logging_folder + "warn.log", std::ios::app | std::ios::out);
    info_stream.open(core_logging_folder + "info.log", std::ios::app | std::ios::out);
    debug_stream.open(core_logging_folder + "debug.log", std::ios::app | std::ios::out);
}

/**
 * Class deconstructor.
 * Closes file descriptors, if open.
 */
Logging::~Logging() {
    close_log_files();
}


/**
 * Closes all open logging file descriptors.
 */
void Logging::close_log_files() {
    if (debug_stream.is_open()) {
        debug_stream.close();
    }
    if (info_stream.is_open()) {
        info_stream.close();
    }
    if (warn_stream.is_open()) {
        warn_stream.close();
    }
    if (error_stream.is_open()) {
        error_stream.close();
    }
}


/**
 * Writes message to info log file.
 */
void Logging::debug(std::string msg, std::string log_type, const char* file, long line_num) {
    // Build and write logging message.
    std::string log_prefix = " [" + log_type + "] [ " + std::string(file) + " | " + std::to_string(line_num) + " ] ";
    debug_stream << log_prefix << msg << std::endl;
}


/**
 * Writes message to info log file.
 */
void Logging::info(std::string msg, std::string log_type, const char* file, long line_num) {
    // Build and write logging message.
    std::string log_prefix = " [" + log_type + "] [ " + std::string(file) + " | " + std::to_string(line_num) + " ] ";
    info_stream << log_prefix << msg << std::endl;

    // Write to console.
    std::cout << log_prefix << msg << std::endl;

    // Also write to lower level loggers.
    debug(msg, log_type, file, line_num);
}

/**
 * Writes message to warning log file.
 */
void Logging::warn(std::string msg, std::string log_type, const char* file, long line_num) {
    // Build and write logging message.
    std::string log_prefix = " [" + log_type + "] [ " + std::string(file) + " | " + std::to_string(line_num) + " ] ";
    warn_stream << log_prefix << msg << std::endl;

    // Also write to lower level loggers.
    info(msg, log_type, file, line_num);
}

/**
 * Writes message to error log file.
 */
void Logging::error(std::string msg, std::string log_type, const char* file, long line_num) {
    // Build and write logging message.
    std::string log_prefix = " [" + log_type + "] [ " + std::string(file) + " | " + std::to_string(line_num) + " ] ";
    error_stream << log_prefix << msg << std::endl;

    // Also write to lower level loggers.
    warn(msg, log_type, file, line_num);
}


/**
 * Prints out logging location to console.
 */
void Logging::display_logging_folder() {
    std::cout << "Logging folder: " << core_logging_folder << std::endl;
}


/**
 * Resets contents of logging files.
 */
void Logging::clear_log_files() {
    // First close any open logging file descriptors.
    close_log_files();

    // Open log files in "write mode", which will override any present data.
    error_stream.open(core_logging_folder + "error.log", std::ios::out);
    warn_stream.open(core_logging_folder + "warn.log", std::ios::out);
    info_stream.open(core_logging_folder + "info.log", std::ios::out);
    debug_stream.open(core_logging_folder + "debug.log", std::ios::out);

    // Close log files and reopen in "append mode". Probably not necessary but here to be safe.
    close_log_files();
    error_stream.open(core_logging_folder + "error.log", std::ios::app | std::ios::out);
    warn_stream.open(core_logging_folder + "warn.log", std::ios::app | std::ios::out);
    info_stream.open(core_logging_folder + "info.log", std::ios::app | std::ios::out);
    debug_stream.open(core_logging_folder + "debug.log", std::ios::out);
}
